package App::ZFSCurses::UI;

=head1 NAME

App::ZFSCurses::UI - Draw UI components.

=cut

=head1 VERSION

Version 1.210.

=cut

our $VERSION = '1.210';

=head1 SYNOPSIS

The App::ZFSCurses::UI namespace is made up of two modules, namely:

=over 2

=item L<App::ZFSCurses::UI::Datasets>

Draw UI components showing a list of ZFS datasets.

=item L<App::ZFSCurses::UI::Snapshots>

Draw UI components showing a list of ZFS snapshots.

=cut

=head1 SEE ALSO

The L<Curses::UI> Perl module.

=head1 AUTHOR

Patrice Clement <monsieurp at cpan.org>

=head1 LICENSE AND COPYRIGHT

This software is copyright (c) 2020 by Patrice Clement.

This is free software, licensed under the (three-clause) BSD License.

See the LICENSE file.

=cut

1;
